package oracle.retail.microapps.map.managed;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;

import oracle.adf.controller.TaskFlowId;
import oracle.adf.model.binding.DCIteratorBinding;
import oracle.adf.share.ADFContext;
import oracle.adf.view.faces.bi.component.geoMap.DataContent;
import oracle.adf.view.faces.bi.event.MapClickActionEvent;
import oracle.adf.view.faces.bi.model.GeoMapDataModel;

import oracle.retail.merch20.common.ADFUtils;
import oracle.retail.microapps.map.config.MapConfig;
import oracle.retail.microapps.map.config.MapConfigReader;
import oracle.retail.microapps.map.managed.helper.InContextAppDispatcher;
import oracle.retail.microapps.map.managed.helper.MapModelDispatcher;

import org.apache.log4j.Logger;

public class MapManager{
    
    private Logger logger = Logger.getLogger(MapManager.class);
    
    /**
     * Object containing the user configuration for the Map. 
     * This configuration is read from an XML file.
     */
    private oracle.retail.microapps.map.config.MapConfig mapConfig;
    
    /**
     * HashMap containing ThemeID as key and corresponding DataModel as value.
     */
    private HashMap<String, GeoMapDataModel> themesMap;
    
    /**
     * HashMap containing ThemeID as key and a Boolean as value. 
     * Value indicates if the theme is selected for rendering.
     */
    private HashMap<String, Boolean> renderTheme;

    /**
     * List containing Apps applicable for currently selected theme,
     * Currently selected theme is one which user clicks on.
     */
    private List<MapConfig.Themes.Theme.LaunchableApps.App> currentAppsList;
    
    /**
     * Object encapsulating all info for the point currently clicked on a theme.
     * This can be used to access info for the point clicked, using EL expression.
     */
    private DataContent themePointDataContent;
    
    /**
     * This represents the task flow ID for the In Context launch Application.
     */
    private String taskFlowId = "/WEB-INF/create-item-flow.xml#create-item-flow";
    
    /**
     * For internal use only. No getter setter.
     */
    private DCIteratorBinding currentThemeIterator;

    public MapManager() {
        super();
        logger.debug("Managed bean MapManager, instantiated");
        readConfig();
    }
    
    private void readConfig(){
        logger.debug("Reading Map configuration");
        mapConfig = MapConfigReader.readConfig();
        prepareMapModel();
    }
    
    private void prepareMapModel(){
        logger.debug("Preparing theme models");
        themesMap = new HashMap<String, GeoMapDataModel>();
        renderTheme = new HashMap<String, Boolean>();
        //getting theme config
        List<MapConfig.Themes.Theme> themeList = mapConfig.getThemes().getTheme();
        for(MapConfig.Themes.Theme theme : themeList){
            logger.debug("Putting model for "+theme.getId());
            themesMap.put(theme.getId(), MapModelDispatcher.dispatchModel(theme));
            if(theme.isRendered() == true){
                logger.debug("Adding "+theme.getId()+" to selected themes for rendering ");
                renderTheme.put(theme.getId(), Boolean.TRUE);
            } else {
                logger.debug("Adding "+theme.getId()+" to selected themes for hiding ");
                renderTheme.put(theme.getId(), Boolean.FALSE);
            }
        }
    }
    
    /**
     * A Map can have several associated themes. A user may select the theme to hide or display
     * by selecting checkbox for associated theme. This method captures the user input from the
     * checkbox value change event, and changes the theme rendering property.
     * @param themeSelectionChangeEvent
     */
    public void themeSelectionChange(ValueChangeEvent themeSelectionChangeEvent){
        logger.debug("Theme selection event ");
        String themeID = (String)themeSelectionChangeEvent.getComponent().getAttributes().get("ThemeID");
        //iterate through the themes, and change rendered property
        List<MapConfig.Themes.Theme> themeList = mapConfig.getThemes().getTheme();
        for(MapConfig.Themes.Theme theme : themeList){
            if(theme.getId().equals(themeID)){
                if(themeSelectionChangeEvent.getNewValue().equals(Boolean.TRUE)){
                    logger.debug("Theme selected for display > "+themeID);
                    theme.setRendered(true);
                    //change the value in the render theme map
                    renderTheme.put(theme.getId(), Boolean.TRUE);
                }
                if(themeSelectionChangeEvent.getNewValue().equals(Boolean.FALSE)){
                    logger.debug("Theme selected for hiding < "+themeID);
                    theme.setRendered(false);
                    //change the value in the render theme map
                    renderTheme.put(theme.getId(), Boolean.FALSE);
                }
            }
        }
    }
    
    /**
     * Each theme in the map may be associated with a number of launchable apps. When the user
     * rightclicks on a data point in the theme, this method captures information about the selected
     * point and populates an App list with the launchable Apps for that theme. It also sets the current
     * row of the iterator to represent the point selected.
     * @param mapClickEvent
     */
    public void clickListener(MapClickActionEvent mapClickEvent){
        String themeClickedOn = mapClickEvent.getSelectionThemeId();
        logger.debug("Clicked on theme "+themeClickedOn);
        currentAppsList = new ArrayList<MapConfig.Themes.Theme.LaunchableApps.App>();   
        List<MapConfig.Themes.Theme> themeList = mapConfig.getThemes().getTheme();
        for(MapConfig.Themes.Theme theme: themeList){
            if(theme.getId().equals(themeClickedOn)){
                //getting App list for the theme
                logger.debug("Adding app list of theme "+themeClickedOn);
                this.currentAppsList = theme.getLaunchableApps().getApp();
                logger.debug("App list now is "+currentAppsList);
                //adding datacontent for the point clicked...
                logger.debug("Adding themePointDataContent ");
                themePointDataContent = mapClickEvent.getDataContent();
                //checking current row for VO
                logger.debug("Setting current row in iterator");
                String viewObjectIter = theme.getIteratorName();
                currentThemeIterator = ADFUtils.findIterator(viewObjectIter);
                try{
                    String iterKey = mapClickEvent.getDataContent().getGeoRowObject().getKey().getRowId().toString();
                    logger.debug("Setting current row of iter with key : "+iterKey);
                    currentThemeIterator.setCurrentRowWithKeyValue(iterKey);
                    logger.debug("Current row is set");
                } catch(Exception e) {
                    logger.debug("Exception while accessing iterator "+viewObjectIter);
                    logger.debug("No parameter passing will occur");
                    logger.debug(e);
                }
                break;
            }
        }
    }
    
    /**
     * When a user clicks to launch an Application, this action listner is called.
     * It determines the ID of the Application to be launched and populates the
     * TaskFlowID value for that Application.
     * @param appClickEvent
     */
    public void appLaunchListner(ActionEvent appClickEvent){
        String appClickedOn = (String)appClickEvent.getComponent().getAttributes().get("AppID");
        logger.debug("Clicked on the App ID "+appClickedOn);
        for(MapConfig.Themes.Theme.LaunchableApps.App app: currentAppsList){
            if(app.getId().equals(appClickedOn)){
                taskFlowId = InContextAppDispatcher.getAppTaskFlowID(appClickedOn);
                //setting configured parameters in pageFlowScope
                List<MapConfig.Themes.Theme.LaunchableApps.App.Parameter> paramList = app.getParameter();
                for(MapConfig.Themes.Theme.LaunchableApps.App.Parameter param: paramList){
                    //getting param name which indicates iter column to fetch from
                    String paramName = param.getIterAttr();
                    //getting param id which would be used to set the param value
                    String paramID = param.getDest();
                    //getting param value from the theme iter
                    String paramValue = currentThemeIterator.getCurrentRow().getAttribute(paramName).toString();
                    logger.debug("Param name configured for the app : "+paramName);
                    logger.debug("Default value configured for the param : "+param.getDefaultValue());
                    logger.debug("Value got from the iterator for the param : "+paramValue);
                    if(paramValue != null)
                        ADFContext.getCurrent().getViewScope().put(paramID, paramValue);
                    else
                        ADFContext.getCurrent().getViewScope().put(paramID, param.getDefaultValue());
                }
            }
        }
    }
    
    public void setMapConfig(MapConfig mapConfig) {
        this.mapConfig = mapConfig;
    }

    public MapConfig getMapConfig() {
        return mapConfig;
    }

    public void setThemesMap(HashMap<String, GeoMapDataModel> themesMap) {
        this.themesMap = themesMap;
    }

    public HashMap<String, GeoMapDataModel> getThemesMap() {
        return themesMap;
    }

    public TaskFlowId getDynamicTaskFlowId() {
        return TaskFlowId.parse(taskFlowId);
    }

    public void setCurrentAppsList(List<MapConfig.Themes.Theme.LaunchableApps.App> currentAppsList) {
        this.currentAppsList = currentAppsList;
    }

    public List<MapConfig.Themes.Theme.LaunchableApps.App> getCurrentAppsList() {
        return currentAppsList;
    }

    public void setRenderTheme(HashMap<String, Boolean> renderTheme) {
        this.renderTheme = renderTheme;
    }

    public HashMap<String, Boolean> getRenderTheme() {
        return renderTheme;
    }

    public void setThemePointDataContent(DataContent themePointDataContent) {
        this.themePointDataContent = themePointDataContent;
    }

    public DataContent getThemePointDataContent() {
        return themePointDataContent;
    }
}
